﻿using SoftwareInstallationContracts.BusinessLogicsContracts;
using SoftwareInstallationContracts.ViewModels;
using SoftwareInstallationDataModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using SoftwareInstallationContracts.BindingModels;

namespace SoftwareInstallationView
{
    public partial class FormShop : Form
    {
        private readonly List<ShopViewModel>? _listShops;
        private readonly IShopLogic _logic;
        private readonly ILogger _logger;

        public int Id
        {
            get; set;
        }

        private IShopModel? GetShop(int id)
        {
            if (_listShops == null)
            {
                return null;
            }
            foreach (var elem in _listShops)
            {
                if (elem.Id == id)
                {
                    return elem;
                }
            }
            return null;
        }
        public FormShop(ILogger<FormShop> logger, IShopLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _listShops = logic.ReadList(null);
            _logic = logic;

        }
        private void LoadData(bool extendDate = true)
        {
            try
            {
                var model = GetShop(extendDate ? Id : Convert.ToInt32(null));
                if (model != null)
                {
                    textBoxShop.Text = model.Name;
                    textBoxAddress.Text = model.Address;
                    openingDatePicker.Text = Convert.ToString(model.DateOpening);
                    dataGridView.Rows.Clear();
                    foreach (var el in model.Packages.Values)
                    {
                        dataGridView.Rows.Add(new object[] { el.Item1.PackageName, el.Item1.Price, el.Item2 });
                    }
                }
                _logger.LogInformation("Загрузка магазинов");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки магазинов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                MessageBoxIcon.Error);
            }
        }

        private void buttonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxShop.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(textBoxAddress.Text))
            {
                MessageBox.Show("Заполните адрес", "Ошибка", MessageBoxButtons.OK,
                MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Сохранение изделия");
            try
            {
                DateTime.TryParse(openingDatePicker.Text, out var dateTime);
                ShopBindingModel model = new()
                {
                    Name = textBoxShop.Text,
                    Address = textBoxAddress.Text,
                    DateOpening = dateTime
                };
                var vmodel = GetShop(Id);
                bool operationResult = false;

                if (vmodel != null)
                {
                    model.Id = vmodel.Id;
                    operationResult = _logic.Update(model);
                }
                else
                {
                    operationResult = _logic.Create(model);
                }
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение",
                MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения изделия");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
        private void FormShop_Load(object sender, EventArgs e)
        {
            LoadData();
        }
    }
}
